import logging
import warnings
from http import HTTPStatus
from pathlib import Path
from typing import List, Optional, Dict, Union
from urllib.parse import urljoin

import httpcore
import httpx
import typer
from PyInquirer import prompt
from tenacity import (
    retry,
    before_sleep_log,
    retry_if_exception_type,
    wait_fixed,
    wait_random,
)
from tqdm import tqdm

from kurby.api import get_auth_client, get_animes
from kurby.constants import (
    FUZZY_SEARCH_THRESHOLD,
    FUZZY_SEARCH_MAX_RESULTS,
    TWIST_CDN_URL,
)
from kurby.constants import TWIST_URL
from kurby.exceptions import MissingEpisodeError
from kurby.schemas import Anime, AnimeSource

warnings.simplefilter("ignore")
from fuzzywuzzy import fuzz

logger = logging.getLogger(__name__)


def filter_animes(
    search: str,
    animes: Optional[List[Anime]] = None,
    threshold: int = FUZZY_SEARCH_THRESHOLD,
    limit: int = FUZZY_SEARCH_MAX_RESULTS,
):
    if animes is None:
        animes = get_animes()
    animes_by_title = dict(
        sorted(((anime.title, anime) for anime in animes), key=lambda x: x[0])
    )
    animes_by_alt_title = dict(
        sorted(
            ((anime.alt_title, anime) for anime in animes if anime.alt_title),
            key=lambda x: x[0],
        )
    )
    selected_animes_with_score_by_id = {}
    for anime in animes_by_title.values():
        score = fuzz.token_set_ratio(search.lower(), anime.title.lower())
        if score > threshold:
            selected_animes_with_score_by_id[anime.id] = (score, anime)
    for anime in animes_by_alt_title.values():
        score = fuzz.token_set_ratio(search.lower(), anime.alt_title.lower())
        if score > threshold:
            selected_animes_with_score_by_id[anime.id] = (score, anime)
    return list(
        x[1]
        for x in sorted(
            selected_animes_with_score_by_id.values(),
            key=lambda x: x[0],
            reverse=True,
        )
    )[:limit]


@retry(
    reraise=True,
    before_sleep=before_sleep_log(logger, logging.ERROR),
    retry=retry_if_exception_type(httpcore.TimeoutException)
    | retry_if_exception_type(httpx.NetworkError)
    | retry_if_exception_type(httpx.TransportError)
    | retry_if_exception_type(httpx.HTTPStatusError),
    wait=wait_fixed(2) + wait_random(1, 10),
)
def download_source(source: AnimeSource, filepath: Path):
    with get_auth_client() as new_client:
        url = urljoin(TWIST_CDN_URL, source.source)
        with new_client.stream(
            "GET",
            url,
            headers={**dict(new_client.headers), "referer": TWIST_URL},
        ) as response:
            if response.status_code == HTTPStatus.NOT_FOUND:
                raise MissingEpisodeError(response=response)
            response.raise_for_status()
            total = int(response.headers.get("Content-Length"))
            with filepath.open("wb") as file:
                with tqdm(
                    total=total, unit_scale=True, unit_divisor=1024, unit="B"
                ) as progress:
                    num_bytes_downloaded = response.num_bytes_downloaded
                    for chunk in response.iter_bytes():
                        file.write(chunk)
                        progress.update(
                            response.num_bytes_downloaded - num_bytes_downloaded
                        )
                        num_bytes_downloaded = response.num_bytes_downloaded


def select_anime_slug(slug: Optional[str]) -> str:
    def get_choices(context: Dict[str, str]) -> List[Dict[str, Union[str, Anime]]]:
        animes = get_animes()
        if context.get("filter"):
            animes = filter_animes(context["filter"], animes=animes)
        return [
            {"name": anime.full_title(), "value": anime}
            for anime in sorted((a for a in animes), key=lambda a: a.title)
        ]

    if slug is None:
        questions = [
            {
                "type": "input",
                "name": "filter",
                "message": "Apply a filter (Press [ENTER] to ignore):",
            },
            {
                "type": "list",
                "name": "anime",
                "message": "Select an anime:",
                "choices": get_choices,
            },
        ]
        answers = prompt(questions)
        if "anime" not in answers:
            raise typer.BadParameter(f"No choice selected.")
        return answers["anime"].slug.slug
    return slug



import base64
import re
from typing import List
from urllib.parse import urljoin

import httpx
import js2py
from pydantic import parse_obj_as

from kurby.constants import TWIST_URL, ANIME_ENDPOINT, DEFAULT_TIMEOUT
from kurby.decrypt import decrypt
from kurby.exceptions import SourceKeyError, ServerError, RequestError
from kurby.schemas import Anime, AnimeDetails, AnimeSource
from kurby.utils import get_chrome_headers


class TwistClient(httpx.Client):
    source_key = None
    access_token = None


def get_source_key(content: str) -> bytes:
    match = re.search(r',k:.*\("(.*?)"\),mount', content)
    if not match:
        raise SourceKeyError
    return base64.b64decode(match.group(1))


def get_auth_client() -> TwistClient:
    headers = get_chrome_headers()
    try:
        r = httpx.get(url=TWIST_URL, headers=headers, timeout=DEFAULT_TIMEOUT)
        r.raise_for_status()
        match = re.search(r"<script>(.*)<\/script>", r.content.decode("utf-8"))
        script = match.group(1)
        script = script.replace("=eval", "=function(a) {return a;}")
        cookie_script = js2py.eval_js(script).replace("location.reload();", "")
        cookie_script = cookie_script.replace("document.cookie=", "value=")
        cookie_script += "func=function(a) {return a;};func(value);"
        cookie = js2py.eval_js(cookie_script).split("=", 1)
        cookies = {cookie[0]: cookie[1]}

        r = httpx.get(
            url=TWIST_URL, headers=headers, cookies=cookies, timeout=DEFAULT_TIMEOUT
        )
        match = re.search(
            r'<script src="(\/_nuxt\/\w+\.js)" type="text\/javascript"><\/script>\s*$',
            r.content.decode("utf-8"),
        )
        script_url = urljoin(TWIST_URL, match.group(1))
        r = httpx.get(
            url=urljoin(TWIST_URL, script_url),
            headers=headers,
            cookies=cookies,
            timeout=DEFAULT_TIMEOUT,
        )
        content = r.content.decode("utf-8")
        r.raise_for_status()
        match = re.search(r'"x-access-token":"([\w]+)"', content)
        access_token = match.group(1)
    except httpx.HTTPStatusError as e:
        raise RequestError(e)
    except httpx.HTTPError as e:
        raise ServerError(e)

    c = TwistClient(
        headers={**headers, "x-access-token": access_token},
        cookies=cookies,
        timeout=DEFAULT_TIMEOUT,
    )
    # Extra parameters
    c.source_key = get_source_key(content)
    c.access_token = access_token
    return c


def get_animes() -> List[Anime]:
    with get_auth_client() as client:
        try:
            r = client.get(url=f"{TWIST_URL}{ANIME_ENDPOINT}")
            r.raise_for_status()
        except httpx.HTTPStatusError as e:
            raise RequestError(e)
        except httpx.HTTPError as e:
            raise ServerError(e)
        return parse_obj_as(List[Anime], r.json())


def get_anime_details(anime: Anime) -> AnimeDetails:
    with get_auth_client() as client:
        url = f"{TWIST_URL}{ANIME_ENDPOINT}/{anime.slug.slug}"
        try:
            r = client.get(url=url)
            r.raise_for_status()
        except httpx.HTTPStatusError as e:
            raise RequestError(e)
        except httpx.HTTPError as e:
            raise ServerError(e)

        anime_details: AnimeDetails = AnimeDetails.parse_obj(r.json())
        return anime_details


def get_sources(anime: Anime) -> List[AnimeSource]:
    with get_auth_client() as client:
        source_key = client.source_key
        url = f"{TWIST_URL}{ANIME_ENDPOINT}/{anime.slug.slug}/sources"
        try:
            r = client.get(url=url)
            r.raise_for_status()
        except httpx.HTTPStatusError as e:
            raise RequestError(e)
        except httpx.HTTPError as e:
            raise ServerError(e)
        sources: List[AnimeSource] = parse_obj_as(List[AnimeSource], r.json())
        # Decrypt and complete source
        for source in sources:
            source.source = (
                decrypt(source.source.encode("utf-8"), source_key)
                .decode("utf-8")
                .replace(" ", "%20")
            )
        return sources



import os
from pathlib import Path

# API
import typer

TWIST_URL = "https://twist.moe"
TWIST_CDN_URL = "https://air-cdn.twist.moe"
ANIME_ENDPOINT = "/api/anime"


DEFAULT_ACCEPT_LANGUAGE_HEADER = "en-US,en;q=0.9"
CHROME_HEADERS = {
    "authority": "twist.moe",
    "pragma": "no-cache",
    "cache-control": "no-cache",
    "sec-ch-ua": '"Google Chrome";v="87", " Not;A Brand";v="99", "Chromium";v="87"',
    "sec-ch-ua-mobile": "?0",
    "dnt": "1",
    "upgrade-insecure-requests": "1",
    "accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9",
    "sec-fetch-site": "none",
    "sec-fetch-mode": "navigate",
    "sec-fetch-user": "?1",
    "sec-fetch-dest": "document",
}
# CLI
ANIMES_COMMAND_NAME = "animes"
DETAILS_COMMAND_NAME = "details"
DOWNLOAD_COMMAND_NAME = "download"
UPDATE_COMMAND_NAME = "update"

ANIME_SLUG_HELP = (
    "Anime slug. Use "
    + typer.style(ANIMES_COMMAND_NAME, bold=True)
    + " command for more information."
)

# MISC
PACKAGE_NAME = "kurby"
CWD_DIR = Path(os.getcwd())
ROOT_DIR = os.path.dirname(Path(__file__).parent)
FUZZY_SEARCH_THRESHOLD = 60
FUZZY_SEARCH_MAX_RESULTS = 10
MAX_SOURCE_TEST_ITERATION = 32
DEFAULT_TIMEOUT = 30.0



import os
import re
import subprocess
import sys
import tempfile
import unicodedata
import webbrowser
from typing import Dict, Optional

import httpx
import pkg_resources
from faker import Faker

from kurby.constants import DEFAULT_ACCEPT_LANGUAGE_HEADER, CHROME_HEADERS, PACKAGE_NAME

fake = Faker()


def open_in_browser(response: httpx.Response):
    content = response.content

    if b"<base" not in content:
        repl = f'<head><base href="{response.url}">'.encode("utf-8")
        content = content.replace(b"<head>", repl)
    ext = ".html"
    fd, fname = tempfile.mkstemp(ext)
    os.write(fd, content)
    os.close(fd)
    return webbrowser.open("file://%s" % fname)


def slugify(value, allow_unicode=False):
    value = str(value)
    if allow_unicode:
        value = unicodedata.normalize("NFKC", value)
    else:
        value = (
            unicodedata.normalize("NFKD", value)
            .encode("ascii", "ignore")
            .decode("ascii")
        )
    value = re.sub(r"[^\w\s]", ".", value).strip()
    return "".join(
        x for x in re.sub(r"[\.\s]+", ".", value) if x.isalnum() or x in "._- "
    )


def get_accept_language_header() -> str:
    # TODO: this is not reliable on all countries
    # try:
    #     local_settings, *_ = locale.getlocale()
    #     language_code, country_code = local_settings.split("_")
    #     return f"{language_code}-{country_code},{language_code};q=0.9"
    # except:
    return DEFAULT_ACCEPT_LANGUAGE_HEADER


def get_chrome_headers() -> Dict[str, str]:
    return {
        **CHROME_HEADERS,
        "accept-language": get_accept_language_header(),
        "user-agent": fake.chrome(),
    }


def get_current_version(package=PACKAGE_NAME) -> str:
    return pkg_resources.get_distribution(package).version


def check_for_update(current_version, package=PACKAGE_NAME) -> Optional[str]:
    try:
        r = httpx.get(f"https://pypi.org/pypi/{package}/json", timeout=1)
    except httpx.HTTPError:
        return
    releases = list(
        sorted(
            (
                {
                    "raw": k,
                    "version": tuple(int(x) for x in k.split("v", 1)[-1].split(".")),
                    "current": bool(k == current_version),
                }
                for k in r.json()["releases"]
            ),
            key=lambda r: r["version"],
            reverse=True,
        )
    )
    if releases and not releases[0]["current"]:
        return releases[0]["raw"]


def install_package(version, package=PACKAGE_NAME):
    subprocess.check_call(
        [sys.executable, "-m", "pip", "install", f"{package}=={version}"]
    )



from pathlib import Path
from typing import Optional, List

import typer

from kurby.helpers import filter_animes
from kurby.schemas import Anime, AnimeDetails, AnimeSource


def invalid_slug_message(slug: str, animes: Optional[List[Anime]] = None):
    suggestions = filter_animes(search=slug, animes=animes, limit=3)
    message = typer.style(slug, fg=typer.colors.RED) + " anime doesn't exist.\n"
    if suggestions:
        message += "The most similar slug(s) are :\n"
        for sug in suggestions:
            message += typer.style(
                f"{sug.slug.slug} \n", fg=typer.colors.GREEN, bold=True
            )

    message += (
        "\nUse " + typer.style("animes", bold=True) + " command to get correct slug."
    )
    return message


def anime_message(anime: Anime):
    return (
        anime.full_title(stylized=True)
        + (f"\n\tSeason: {anime.season}" if anime.season else "")
        + (typer.style(" - Ongoing", fg=typer.colors.YELLOW) if anime.season else "")
        + "\n\tSlug: "
        + typer.style(anime.slug.slug, fg=typer.colors.GREEN)
        + "\n"
    )


def anime_details_message(anime_details: AnimeDetails):
    sorted_episodes = list(sorted(anime_details.episodes, key=lambda ep: ep.number))
    message = (
        typer.style(f"{anime_details.title}", bold=True)
        + f"{f' ({anime_details.alt_title})' if anime_details.alt_title else ''}"
        + (f"\n\tSeason: {anime_details.season}" if anime_details.season else "")
        + (typer.style(" - Ongoing", blink=True) if anime_details.season else "")
        + f"\n{anime_details.description or ''}\n"
    )
    if sorted_episodes:
        message += (
            "\n Episodes going from "
            + typer.style(str(sorted_episodes[0].number), bold=True)
            + " to "
            + typer.style(str(sorted_episodes[-1].number), bold=True)
            + "\n First episode uploaded at "
            + typer.style(sorted_episodes[0].created_at.isoformat(sep=" "), bold=True)
            + "\n Last episode uploaded at "
            + typer.style(sorted_episodes[-1].created_at.isoformat(sep=" "), bold=True)
        )
    return message


def download_starting_message(sources: List[AnimeSource], directory: Path):
    return (
        f"Downloading "
        + typer.style(str(len(sources)), bold=True)
        + " files into "
        + typer.style(str(directory.absolute()), bold=True, fg=typer.colors.GREEN)
        + "..."
    )



import base64

from hashlib import md5
from Cryptodome.Cipher import AES

BLOCK_SIZE = 16

"""
Code used to replicate CryptoJS decryption
"""


def unpad(ciphertext: bytes) -> bytes:
    return ciphertext[
        : -(ciphertext[-1] if type(ciphertext[-1]) == int else ord(ciphertext[-1]))
    ]


def bytes_to_aes_key(data: bytes, salt: bytes, output=48) -> bytes:
    assert len(salt) == 8, len(salt)

    data += salt
    key = md5(data).digest()
    final_key = key

    while len(final_key) < output:
        key = md5(key + data).digest()
        final_key += key

    return final_key[:output]


def decrypt(encrypted: bytes, passphrase: bytes) -> bytes:
    encrypted = base64.b64decode(encrypted)
    assert encrypted[0:8] == b"Salted__"

    salt = encrypted[8:16]
    key_iv = bytes_to_aes_key(passphrase, salt, 32 + 16)
    key = key_iv[:32]
    iv = key_iv[32:]
    aes = AES.new(key, AES.MODE_CBC, iv)

    return unpad(aes.decrypt(encrypted[16:]))



import httpx


class KurbyError(Exception):
    code = "undefined"
    message = "An unknown error occurred."

    def __init__(self):
        super().__init__(self.message)

    def __str__(self):
        return f"({self.code}): {self.message}"


class CrawlingError(KurbyError):
    code = "crawling_error"
    message = "An unknown crawling error occurred."


class SourceKeyError(CrawlingError):
    code = "source_key_error"
    message = "Couldn't get the source key."


class ServerError(CrawlingError):
    code = "server_error"
    message = "Twist didn't respond correctly. You might need to try again later."

    def __init__(self, exc: httpx.HTTPError):
        self.message += f"\n{exc.__cause__}"


class RequestError(CrawlingError):
    code = "request_error"
    message = "Something bad happened when requesting Twist. You might need update Kurby or to try again later."

    def __init__(self, exc: httpx.HTTPStatusError):
        self.message += f"\nError code {exc.response.status_code} while requesting {exc.request.url!r}."


class MissingEpisodeError(KurbyError):
    code = "missing_episode"
    message = "An episode is missing."

    def __init__(self, response: httpx.Response):
        self.response = response



import datetime
from typing import Optional, List

import typer
from pydantic import BaseModel


class AnimeSlug(BaseModel):
    id: int
    slug: str
    anime_id: int
    created_at: datetime.datetime
    updated_at: datetime.datetime


class Anime(BaseModel):
    id: int
    title: str
    alt_title: Optional[str]
    season: int
    ongoing: int
    hb_id: Optional[int]
    created_at: datetime.datetime
    updated_at: datetime.datetime
    hidden: bool
    mal_id: Optional[int]
    slug: AnimeSlug

    def full_title(self, stylized=False) -> str:
        alt_title = f"{f' ({self.alt_title})' if self.alt_title else ''}"
        if stylized:
            return typer.style(f"{self.title}", bold=True) + alt_title
        return f"{self.title}{alt_title}"


class AnimeEpisode(BaseModel):
    id: int
    number: int
    anime_id: int
    created_at: datetime.datetime
    updated_at: datetime.datetime


class AnimeDetails(Anime):
    description: Optional[str]
    episodes: List[AnimeEpisode]


class AnimeSource(BaseModel):
    id: int
    source: str
    number: int
    anime_id: int
    created_at: datetime.datetime
    updated_at: datetime.datetime



#!/usr/bin/env python

import datetime
from distutils.util import strtobool
import inspect
import os
from functools import wraps
from pathlib import Path
from typing import Optional, Callable

import typer

from kurby.constants import (
    ANIME_SLUG_HELP,
    CWD_DIR,
    ANIMES_COMMAND_NAME,
    DETAILS_COMMAND_NAME,
    DOWNLOAD_COMMAND_NAME,
    UPDATE_COMMAND_NAME,
)
from kurby.exceptions import KurbyError, MissingEpisodeError
from kurby.helpers import (
    download_source,
    filter_animes,
    select_anime_slug,
)
from kurby.api import get_animes, get_anime_details, get_sources
from kurby.messages import (
    invalid_slug_message,
    anime_message,
    anime_details_message,
    download_starting_message,
)
from kurby.utils import slugify, check_for_update, get_current_version, install_package

app = typer.Typer(
    help="""
    A nice CLI to download animes from twist.moe
    
    The developer or this application do not store any animes whatsoever
"""
)

payload = {}
DEBUG = bool(strtobool(os.getenv("KURBY_DEBUG", default="0")))


def cli_exception_handler(func: Callable) -> Callable:
    @wraps(func)
    def wrapper(*args, **kwargs):
        try:
            func(*args, **kwargs)
        except KurbyError as e:
            typer.echo(
                "Error "
                + typer.style(e.code, fg=typer.colors.RED, bold=True)
                + f": {e.message}"
            )
            if DEBUG:
                raise e
            raise typer.Exit(1)

    signature = inspect.signature(func)
    setattr(wrapper, "__signature__", signature)

    return wrapper


def start():
    app()


@app.callback()
@cli_exception_handler
def main(
    check_updates: bool = typer.Option(True, help="Check for Kurby updates"),
):
    if check_updates:
        current_version = get_current_version()
        version = check_for_update(current_version=current_version)
        if version:
            typer.echo(
                "You are using Kurby "
                + typer.style(current_version, bold=True, fg=typer.colors.RED)
                + " while version "
                + typer.style(version, bold=True, fg=typer.colors.GREEN)
                + " is available.\nUse "
                + typer.style(UPDATE_COMMAND_NAME, bold=True)
                + " command to update to the latest version !\n"
            )


@app.command(name=ANIMES_COMMAND_NAME)
@cli_exception_handler
def display_animes(
    search: Optional[str] = typer.Option(None, help="Filter results with fuzzy search")
):
    """
    Search and display information about available animes and optionally use --search to fuzzy search

    Among the information given, the slug is what is used in other commands
    """
    animes = get_animes()
    if search:
        animes = filter_animes(search, animes)

    for anime in animes:
        typer.echo(anime_message(anime))


@app.command(name=DETAILS_COMMAND_NAME)
@cli_exception_handler
def display_anime_details(
    slug: str = typer.Argument(None, help=ANIME_SLUG_HELP, callback=select_anime_slug),
):
    """
    Give more details on a specific anime like the number of episodes from a given anime slug
    """
    animes = get_animes()
    animes_by_slug = {anime.slug.slug: anime for anime in animes}
    try:
        anime = animes_by_slug[slug]
    except KeyError:
        raise typer.BadParameter(invalid_slug_message(slug=slug, animes=animes))
    typer.echo(anime_details_message(get_anime_details(anime)))


@app.command(name=DOWNLOAD_COMMAND_NAME)
@cli_exception_handler
def download(
    slug: str = typer.Argument(None, help=ANIME_SLUG_HELP, callback=select_anime_slug),
    directory: str = typer.Option(
        CWD_DIR, "--d", help="Directory where files will be uploaded"
    ),
    nfrom: int = typer.Option(
        None, "--nfrom", help="Select episodes greater or equal to the given number"
    ),
    nto: int = typer.Option(
        None, "--nto", help="Select episodes lesser or equal to the given number"
    ),
    dfrom: datetime.datetime = typer.Option(
        None, "--dfrom", help="Select episodes uploaded after the given date"
    ),
    dto: datetime.datetime = typer.Option(
        None, "--dto", help="Select episodes uploaded before the given date"
    ),
):
    """
    Download a list of episodes from a given anime slug

    The output directory can be specified
    """
    directory = Path(directory)
    animes = get_animes()
    animes_by_slug = {anime.slug.slug: anime for anime in animes}
    try:
        anime = animes_by_slug[slug]
    except KeyError:
        raise typer.BadParameter(invalid_slug_message(slug=slug, animes=animes))
    sources = get_sources(anime)
    filtered_source_ids = set(source.id for source in sources)
    if nfrom:
        if type(nfrom) is int:
            filtered_source_ids -= set(
                source.id for source in sources if source.number < nfrom
            )
    if dfrom:
        filtered_source_ids -= set(
            source.id for source in sources if source.created_at < dfrom
        )
    if nto:
        filtered_source_ids -= set(
            source.id for source in sources if source.number > nto
        )

    if dto:
        filtered_source_ids -= set(
            source.id for source in sources if source.created_at > dto
        )

    sources = list(
        sorted(
            (source for source in sources if source.id in filtered_source_ids),
            key=lambda s: s.number,
        )
    )
    if not sources:
        typer.secho("No sources to download ! ", bold=True)
        raise typer.Exit(code=1)
    typer.echo(download_starting_message(sources=sources, directory=directory))
    missing_episodes = []
    for source in sources:
        typer.echo(f"{anime.title} - S{anime.season:02d} - E{source.number:02d}")
        title_slug = slugify(anime.title)
        current_dir = directory / title_slug
        if not current_dir.exists():
            current_dir.mkdir(parents=True, exist_ok=True)
        ext = source.source.rsplit(".", 1)[-1]
        filepath = (
            current_dir / f"{title_slug}-S{anime.season:02d}-E{source.number:03d}.{ext}"
        )
        try:
            download_source(source, filepath)
        except MissingEpisodeError as e:
            typer.secho(
                f"Couldn't find the episode on Twist ({e.response.url}). Skipping...",
                fg=typer.colors.RED,
                bold=True,
            )
            missing_episodes.append(source.number)
    if missing_episodes:
        typer.secho(f"The following episodes were missing on Twist:\n", bold=True)
        for ep in missing_episodes:
            typer.echo(ep)


@app.command(name=UPDATE_COMMAND_NAME)
def update():
    """
    Update Kurby to the latest version
    """
    try:
        current_version = get_current_version()
        version = check_for_update(current_version=current_version)
        if version:
            install_package(version)
        else:
            typer.echo("You are using the latest version !")
    except Exception:
        typer.secho(
            "The installation of Kurby failed. Please reinstall it manually.",
            fg=typer.colors.RED,
        )
        raise


if __name__ == "__main__":
    start()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['kurby']

package_data = \
{'': ['*']}

install_requires = \
['Faker>=5,<6',
 'Js2Py>=0,<1',
 'PyInquirer>=1.0.3,<2.0.0',
 'arrow>=0.17,<0.18',
 'fuzzywuzzy>=0,<1',
 'httpx>=0,<1',
 'pycryptodomex>=3.9,<4.0',
 'pydantic>=1.7,<2.0',
 'tenacity>=6,<7',
 'tqdm>=4,<5',
 'typer-cli>=0,<1',
 'typer>=0,<1']

entry_points = \
{'console_scripts': ['kurby = kurby.cli:start']}

setup_kwargs = {
    'name': 'kurby',
    'version': '0.3.2',
    'description': 'A modern CLI to download animes automatically from Twist.moe',
    'long_description': '# Kurby\n[![PyPI - Python Version](https://img.shields.io/badge/python-3.7%20%7C%203.8%20%7C%203.9-blue)](https://docs.python.org/3/whatsnew/3.7.html) [![Downloads](https://pepy.tech/badge/kurby)](https://pepy.tech/badge/kurby) [![PyPI version](https://badge.fury.io/py/kurby.svg)](https://badge.fury.io/py/kurby.svg)\n\n<div align="center">\n    <img src="docs/kurby.png" alt="Kirby ball in Kirby: Canvas Curse" height=400, width=400/>\n</div>\n<br>\n\n\nKurby is a nice and simple CLI that use [Twist](https://twist.moe) website, and their huge collection to download animes for free and **automatically**\n\nAnimes from Twist are generally in High Definition with English subtitles. Please consider [donating](https://twist.moe) for their amazing work :)\n\n## Overview\nThe CLI is built with [Typer](https://github.com/tiangolo/typer) and it is composed of 3 commands\n\n- `animes`: Search animes to find what is available and extract the slug to be used in the other commands\n\n![animes](docs/examples/animes.gif)\n  \n> `--search` option allows you to make a fuzzy search\n  \n- `details`: Show details of an anime if needed\n\n![details](docs/examples/details.gif)\n  \n> You can see the number of episodes available and when the first and last episode were uploaded\n  \n- `download`: Choose and download which episodes of the anime you want !\n\n![download](docs/examples/download.gif)\n\n> Because sometimes bad things can happen, Kurby will automatically retry for you\n\nYou can also download without having a slug initially. In this case, Kurby will propose you a selection\n\n![download-selection](docs/examples/download-selection.gif)\n\n#### And that\'s it !\n\nYou can also use `kurby --help` to get more information on the different commands\n\n## Installation\n```bash\npip install kurby\nkurby --help\n```\n\n## Installation on Windows\n- Right click on the `install.bat` and run it as an **Administrator**, this will install [Chocolatey](https://chocolatey.org/) who will manage the python installation\n- Once the installation is finished, and you\'re asked to press a key, open a new terminal (`Win + R` > `cmd` )\n- You can now start using Kurby with `kurby --help`\n\n## Installation on Android without root needed\n- Install [Termux](https://play.google.com/store/apps/details?id=com.termux) on your Android phone\n- In Termux, run this command to allow access to storage: `termux-setup-storage`, and tap allow\n- Next, run the follow command `pkg install git python`\n- Then `pip3 install kurby`\n- You\'re done ! You can download animes on your phone like so `kurby download --d ~/storage/shared`\n\n##### *Thanks to [6b86b3ac03c167320d93](https://www.reddit.com/user/6b86b3ac03c167320d93/) for this tutorial*\n\n## Installation from sources\n### Create your virtual environment (optional)\n````bash\nmkvirtualenv kurby\nworkon kurby\n````\n### Install poetry\n```bash\ncurl -sSL https://raw.githubusercontent.com/python-poetry/poetry/master/get-poetry.py | python -\n```\n### Install dependencies using poetry\n```bash\npoetry install --no-dev\nkurby-cli --help # or python kurby-cli --help\n```\n\n## Getting the latest episode automatically\nAn interesting use case is to get the latest episode of an anime as soon as it is available.\n\nThis is possible on Linux using `crontab` (or another equivalent on others OS) and _a little tweak of chemistry_.\nHere is an example of a few instructions that can help you do this.\n```shell\nnow=$(date -u "+%Y-%m-%dT%H:%M:%S")\ndate=$(cat kurby.date || echo $now) # Get the date of previous download\npython kurby download {{YOUR_ANIME}} --dfrom=${DATE} # Download any episodes that has been uploaded since the last time\necho $now > kurby.date # Store the current date as the new date\n```\n\n## Next steps\nKurby is already functional as it is but here are the next things I would like to add :\n- Adding the support of asynchronous download\n- Refactor the retrying strategy to add more customisable options and allow errors during a download without interruption\n- Refactor the crawling process to potentially avoid being detected as a bot\n\n### Disclaimer\nDownloading copyright videos may be illegal in your country.\n\nThis tool is for educational purposes only.\n\nThe developers or this application do not store or distribute any files whatsoever.\n',
    'author': 'Alain BERRIER',
    'author_email': 'alain.berrier@outlook.com',
    'maintainer': 'Alain BERRIER',
    'maintainer_email': 'alain.berrier@outlook.com',
    'url': 'https://github.com/aberrier/kurby',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
